<?php
/*
 * Plugin Name: CardsShield Gateway Airwallex
 * Plugin URI:
 * Description: CardsShield Gateway Airwallex
 * Author: CardsShield
 * Author URI: https://cardsshield.com
 * Version: 1.0.4
 *
 /*
 * This action hook registers our PHP class as a WooCommerce payment gateway
 */
if ( !in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
    return;
}

if (!defined('ABSPATH')) {
    exit;
}
register_activation_hook(__FILE__, 'mecom_gateway_airwallex_install');

require_once(plugin_dir_path(__FILE__) . 'utils.php');


if (!class_exists('CSAirwallexUpdateChecker') && is_admin()) {

    class CSAirwallexUpdateChecker
    {

        public $plugin_slug;
        public $version;
        public $cache_key;
        public $cache_allowed;

        public function __construct()
        {

            $this->plugin_slug = plugin_basename(__DIR__);
            $this->version = OPT_MECOM_AIRWALLEX_VERSION;
            $this->cache_key = 'cs_airwallex_update_checker';
            $this->cache_allowed = true;

            add_filter('plugins_api', [$this, 'info'], 20, 3);
            add_filter('site_transient_update_plugins', [$this, 'update']);
            add_action('upgrader_process_complete', [$this, 'purge'], 10, 2);

        }

        public function request()
        {

            $remote = get_transient($this->cache_key);

            if (false === $remote || !$this->cache_allowed) {

                $remote = wp_remote_get(
                    'https://update.cardsshield.com/cs_airwallex/info.php?site=' . urlencode(get_site_url()),
                    [
                        'timeout' => 10,
                        'headers' => [
                            'Accept' => 'application/json'
                        ]
                    ]
                );

                if (
                    is_wp_error($remote)
                    || 200 !== wp_remote_retrieve_response_code($remote)
                    || empty(wp_remote_retrieve_body($remote))
                ) {
                    return false;
                }

                set_transient($this->cache_key, $remote, HOUR_IN_SECONDS);

            }

            $remote = json_decode(wp_remote_retrieve_body($remote));

            return $remote;

        }


        function info($res, $action, $args)
        {

            // do nothing if you're not getting plugin information right now
            if ('plugin_information' !== $action) {
                return $res;
            }

            // do nothing if it is not our plugin
            if ($this->plugin_slug !== $args->slug) {
                return $res;
            }

            // get updates
            $remote = $this->request();

            if (!$remote) {
                return $res;
            }

            $res = new stdClass();

            $res->name = $remote->name;
            $res->slug = $remote->slug;
            $res->version = $remote->version;
            $res->tested = $remote->tested;
            $res->requires = $remote->requires;
            $res->author = $remote->author;
            $res->author_profile = $remote->author_profile;
            $res->download_link = $remote->download_url;
            $res->trunk = $remote->download_url;
            $res->requires_php = $remote->requires_php;
            $res->last_updated = $remote->last_updated;

            $res->sections = [
                'description' => $remote->sections->description,
                'installation' => $remote->sections->installation,
                'changelog' => $remote->sections->changelog
            ];

            if (!empty($remote->banners)) {
                $res->banners = [
                    'low' => $remote->banners->low,
                    'high' => $remote->banners->high
                ];
            }

            return $res;

        }

        public function update($transient)
        {

            if (empty($transient->checked)) {
                return $transient;
            }

            $remote = $this->request();

            if (
                $remote
                && version_compare($this->version, $remote->version, '<')
                && version_compare($remote->requires, get_bloginfo('version'), '<')
                && version_compare($remote->requires_php, PHP_VERSION, '<')
            ) {
                $res = new stdClass();
                $res->slug = $this->plugin_slug;
                $res->plugin = plugin_basename(__FILE__); // misha-update-plugin/misha-update-plugin.php
                $res->new_version = $remote->version;
                $res->tested = $remote->tested;
                $res->package = $remote->download_url;

                $transient->response[$res->plugin] = $res;

            }

            return $transient;

        }

        public function purge($upgrader_object, $options)
        {

            if (
                $this->cache_allowed
                && 'update' === $options['action']
                && 'plugin' === $options['type']
            ) {
                // just clean the cache when new plugin version is installed
                delete_transient($this->cache_key);
            }

        }


    }

    new CSAirwallexUpdateChecker();

}

//Cron
add_filter('cron_schedules', 'mecom_add_airwallex_cron_interval');
if (!wp_next_scheduled('mecom_gateway_airwallex_rotation')) {
    wp_schedule_event(time(), 'one_minute', 'mecom_gateway_airwallex_rotation');
}

add_action('mecom_gateway_airwallex_rotation', 'mecom_airwallex_rotation_checker');
if (!wp_next_scheduled('mecom_gateway_airwallex_daily')) {
    wp_schedule_event(strtotime('23:59:59'), 'daily', 'mecom_gateway_airwallex_daily');
}
add_action('mecom_gateway_airwallex_daily', 'mecom_gateway_airwallex_daily_process');

function mecom_gateway_airwallex_daily_process()
{
    // Reset paid amount
    $rotationMethod = get_option(OPT_MECOM_AIRWALLEX_ROTATION_METHOD, MECOM_AIRWALLEX_BY_TIME);
    if ($rotationMethod === MECOM_AIRWALLEX_BY_AMOUNT) {
        resetPaidAmountAirwallex();
    }
}

add_action('plugins_loaded', 'mecom_add_gateway_airwallex_init');
add_action('get_header', 'handle_airwallex_route');

function mecom_add_airwallex_cron_interval($schedules)
{
    $schedules['one_minute'] = array(
        'interval' => 60,
        'display' => esc_html__('Every minute'),
    );

    return $schedules;
}

function mecom_gateway_airwallex_install()
{
//    delete_option('mecom_gateway_airwallex_version');
//    add_option('mecom_gateway_airwallex_version', uniqid());
}

function renderMoneyRowAirwallex($title, $tooltip, $value, $currency, $negative = false)
{
    /**
     * Bad type hint in WC phpdoc.
     *
     * @psalm-suppress InvalidScalarArgument
     */
    return '
            <tr>
                <td class="label">' . wc_help_tip($tooltip) . ' ' . esc_html($title) . '
                </td>
                <td width="1%"></td>
                <td class="total">
                    ' .
        ($negative ? ' - ' : '') .
        wc_price($value, array('currency' => $currency)) . '
                </td>
            </tr>';
}

function mecom_airwallex_rotation_checker()
{

    $rotationMethod = get_option(OPT_MECOM_AIRWALLEX_ROTATION_METHOD, MECOM_AIRWALLEX_BY_TIME);
    if ($rotationMethod != MECOM_AIRWALLEX_BY_TIME) {
        return;
    }
    // Auto Switching Proxy
    $proxies = get_option(Opt_Mecom_Airwallex_Proxies, []);
    if (empty($proxies)) {
        return;
    }
    $activatedProxy = get_option(Opt_Mecom_Airwallex_Activated_Proxy, null);
    // if only has 1 proxy, don't rotate
    if (count($proxies) === 1 && isset($activatedProxy['id']) && $activatedProxy['id'] === $proxies[0]['id']) {
        return;
    }
    $lastActivatedTimestamp = get_option(OPT_MECOM_AIRWALLEX_CURRENT_ROTATION_VALUE, 0);

    $hasDeletedActivateProxy = false;

    if ($lastActivatedTimestamp != 0 && !empty($activatedProxy)) {
        $currentTimestamp = time();
        $timeStampSinceLastActivated = $currentTimestamp - $lastActivatedTimestamp;
        // How many minutes?
        $minutes = $timeStampSinceLastActivated / 60;

        // Need to rotate
        if ($minutes > floatval($activatedProxy["timestamp"])) {
            $hasDeletedActivateProxy = true;
            for ($i = 0; $i < count($proxies); $i++) {
                $proxy = $proxies[$i];
                if ($proxy["id"] == $activatedProxy["id"]) {
                    // The last one => active the proxy at 0 index
                    if ($i + 1 == count($proxies)) {
                        $needToActivateProxy = $proxies[0];
                    } else {
                        // Active the next one
                        $needToActivateProxy = $proxies[$i + 1];
                    }
                    update_option(Opt_Mecom_Airwallex_Activated_Proxy, $needToActivateProxy, true);
                    update_option(OPT_MECOM_AIRWALLEX_CURRENT_ROTATION_VALUE, time(), true);
                    logAirwallexRotation(MECOM_AIRWALLEX_BY_TIME, $needToActivateProxy, "Auto");
                    $hasDeletedActivateProxy = false;
                    break;
                }
            }
        }
    }

    if (empty($activatedProxy) || $hasDeletedActivateProxy) {
        update_option(Opt_Mecom_Airwallex_Activated_Proxy, $proxies[0], true);
        update_option(OPT_MECOM_AIRWALLEX_CURRENT_ROTATION_VALUE, time(), true);
        logAirwallexRotation(MECOM_AIRWALLEX_BY_TIME, $proxies[0], "Auto");
    }
}

function handle_airwallex_route()
{
    if (isset($_GET['cs_airwallex_complete_order']) && isset($_GET['order_id'])) {
        $order = wc_get_order($_GET['order_id']);
        if (!$order) {
            wc_add_notice('We cannot process your payment right now, please try another payment method.[31]', 'error');
            return wp_redirect(wc_get_checkout_url());
        }
        $activatedProxy = findActivatedProxyDataByIdAirwallex(get_option(Opt_Mecom_Airwallex_Proxies, []), WC()->session->get('mecom-airwallex-proxy-active-id'));

        if (!$activatedProxy) {
            airwallexLog("Can't find activated proxy!\n");
            wc_add_notice('We cannot accept any payments right now. Please comeback to try tomorrow or select other payment methods if available.', 'error');
            return wp_redirect(wc_get_checkout_url());
        }
        $response = wp_remote_post($activatedProxy['url'] . '?mecom-airwallex-confirm-payment-completed=1', [
            'sslverify' => false,
            'timeout' => 5 * 60,
            'headers' => [
                'Content-Type' => 'application/json',
            ],
            'body' => json_encode([
                'payment_intent_id' => $order->get_transaction_id(),
                'merchant_site' => get_home_url(),
                'amount' => $order->get_total(),
                'currency' => $order->get_currency(),
            ])
        ]);
        update_post_meta($order->get_id(), METAKEY_AIRWALLEX_PROXY_URL, $activatedProxy['url']);

        if (is_wp_error($response) || 200 !== wp_remote_retrieve_response_code($response)) {
            airwallexLog($response, 'Airwallex payment error');
            wc_add_notice('We cannot process your payment right now, please try another payment method.[32]', 'error');
            return wp_redirect(wc_get_checkout_url());
        }
        $body = wp_remote_retrieve_body($response);
        $body = json_decode($body);

        if ($body->status === 'success') {
            $paymentIntent = $body->payment;
            $order->payment_complete();
            //Save the processed proxy for this order (using for refund later)
            $order->add_order_note(sprintf(__('Airwallex charged by proxy %s', 'mecom'), $activatedProxy['url']), 0, false);
            // some notes to customer (replace true with false to make it private)
            $order->add_order_note(sprintf(__('Airwallex Checkout charge complete (Payment Intent ID: %s)', 'mecom'), $paymentIntent->id));
            $order->reduce_order_stock();

            if (isEnabledAmountRotationAirwallex()) {
                performProxyAmountRotationAirwallex($order->get_total());
                updateRotationAmountAirwallex($activatedProxy['id'], $order->get_total());
            }

            update_post_meta($order->get_id(), '_transaction_id', $paymentIntent->id);
            // Empty cart
            WC()->cart->empty_cart();
            update_post_meta($order->get_id(), METAKEY_AIRWALLEX_PROXY_URL_ERROR, null);
            return wp_redirect($order->get_checkout_order_received_url());
        } else {
            airwallexLog($response, 'Airwallex confirm payment error');
            // Empty cart
            $order->update_status('failed');
            $err = $body->err;
            $paymentIntentId = $order->get_transaction_id();
            if (isset($err->payment_intent)) {
                $paymentIntentId = $err->payment_intent->id;
                update_post_meta($order->get_id(), '_transaction_id', $paymentIntentId);
                update_post_meta($order->get_id(), METAKEY_AIRWALLEX_PROXY_URL_ERROR, $activatedProxy['url']);
            }
            $order->add_order_note(sprintf(__('Airwallex charged ERROR by proxy %s, ERROR message: %s, Payment Intent ID: %s (3ds verification)', 'mecom'),
                $activatedProxy['url'],
                is_string($err) ?: $err->message,
                $paymentIntentId
            ));
            wc_add_notice('We cannot process your payment right now, please try another payment method.[12]', 'error');
            return wp_redirect(wc_get_checkout_url());
        }
    }
    
    if (isset($_GET['cs_airwallex_fail_order']) && isset($_GET['order_id'])) {
        $order = wc_get_order($_GET['order_id']);
        if ($order && WC()->session->get('mecom-airwallex-last-order-id') == $_GET['order_id']) {
            $activatedProxy = findActivatedProxyDataByIdAirwallex(get_option(Opt_Mecom_Airwallex_Proxies, []), WC()->session->get('mecom-airwallex-proxy-active-id'));
            $order->add_order_note(sprintf(__('Airwallex charged ERROR by proxy %s, ERROR code: %s, ERROR message: %s, Payment Intent ID: %s', 'mecom'),
                $activatedProxy['url'],
                $_GET['code'],
                $_GET['message'],
                $_GET['payment_intent_id']
            ));
            $order->update_status('failed');
        }
    }
}


function mecom_add_gateway_airwallex_init()
{

    if (is_admin()) {
        require_once("m-ecom-airwallex-options.php");
    }

    if (!class_exists('WC_MEcom_Airwallex')) :

        class WC_MEcom_Airwallex
        {

            /**
             * @var Singleton The reference the *Singleton* instance of this class
             */
            private static $instance;

            /**
             * Returns the *Singleton* instance of this class.
             *
             * @return Singleton The *Singleton* instance.
             */
            public static function get_instance()
            {
                if (null === self::$instance) {
                    self::$instance = new self();
                }
                return self::$instance;
            }

            /**
             * Private clone method to prevent cloning of the instance of the
             * *Singleton* instance.
             *
             * @return void
             */
            private function __clone()
            {
            }

            /**
             * Private unserialize method to prevent unserializing of the *Singleton*
             * instance.
             *
             * @return void
             */
            public function __wakeup()
            {
            }

            /**
             * Protected constructor to prevent creating a new instance of the
             * *Singleton* via the `new` operator from outside of this class.
             */
            private function __construct()
            {
                $this->init();
            }

            private function init()
            {
                add_filter('woocommerce_payment_gateways', array($this, 'add_gateways'));
                if (is_admin()) {
                    add_filter(
                        'plugin_action_links_' . plugin_basename(__FILE__),
                        array($this, 'add_settings_link')
                    );
                }
                add_filter('woocommerce_available_payment_gateways', array($this, 'check_cs_airwallex_payment_gateways'), 10, 1);
            }

            public function check_cs_airwallex_payment_gateways($gateways)
            {
                if (!is_checkout() && !defined('WOOCOMMERCE_CHECKOUT')) {
                    return $gateways;
                }

                $rotationMethod = get_option(OPT_MECOM_AIRWALLEX_ROTATION_METHOD, MECOM_AIRWALLEX_BY_TIME);
                global $woocommerce;

                if ($rotationMethod == MECOM_AIRWALLEX_BY_AMOUNT) {
                    $carTotal = WC()->cart->get_total(false);
                    if (!hasPayableProxyAirwallex($carTotal)) {
                        unset($gateways['mecom_airwallex']);
                    }
                }

                return $gateways;
            }

            /**
             * Add the gateways to WooCommerce.
             *
             * @since 1.0.0
             * @version 4.0.0
             */
            public function add_gateways($gateways)
            {
                $gateways[] = 'WC_MEcom_Gateway_Airwallex'; // your class name is here
                return $gateways;
            }

            public function add_settings_link($links)
            {
                $settings = array(
                    'settings' => sprintf(
                        '<a href="%s">%s</a>',
                        admin_url('admin.php?page=wc-settings&tab=checkout&section=mecom_airwallex'),
                        'Settings'
                    )
                );
                return array_merge($settings, $links);
            }
        }

        WC_MEcom_Airwallex::get_instance();

        if (!class_exists('WC_Payment_Gateway')) {
            return;
        }

        class WC_MEcom_Gateway_Airwallex extends WC_Payment_Gateway
        {
            /**
             * Whether or not logging is enabled
             *`
             * @var bool
             */
            public static $log_enabled = false;
            public static $mecom_airwallex_is_inited = false;

            /**
             * Logger instance
             *
             * @var WC_Logger
             */
            public static $log = false;

            /**
             * @var string
             */
            public $productTitleSetting = 'last_word';

            /**
             * @var string
             */
            public $invoice_prefix;

            /**
             * Class constructor, more about it in Step 3
             */
            public function __construct()
            {
                $this->id = 'mecom_airwallex'; // payment gateway plugin ID
                $this->icon = ''; // URL of the icon that will be displayed on checkout page near your gateway name
                $this->has_fields = true; // in case you need a custom credit card form
                $this->method_title = 'CardsShield Gateway Airwallex';
                $this->order_button_text = __('Place order', 'woocommerce');
                $this->method_description = 'CardsShield Gateway Airwallex'; // will be displayed on the options page
                $this->invoice_prefix = $this->get_option('invoice_prefix');

                // gateways can support subscriptions, refunds, saved payment methods,
                // but in this tutorial we begin with simple payments
                $this->supports = array(
                    'products',
                    'refunds',
                );

                // Load the settings.
                $this->init_form_fields();
                $this->init_settings();

                // Define user set variables.
                $this->enabled = $this->get_option('enabled');
                $this->title = $this->get_option('title');
                $this->description = $this->get_option('description');
                $this->payment_notes = $this->get_option('payment_notes');
                $this->testmode = $this->get_option('testmode') === "yes";
                // self::$log_enabled    = $this->debug;

                if ($this->testmode) {
                    /* translators: %s: Link to Airwallex sandbox testing guide page */
                    $this->description .= ' ' . sprintf(__('TEST MODE ENABLED. check the <a href="%s" target="_blank">Testing Airwallex documentation</a> for more card numbers.', 'woocommerce-gateway-airwallex'), 'https://airwallex.com/docs/testing');
                }

                $this->description .= ' ' . $this->payment_notes;
                $this->description = trim($this->description);
                if (!self::$mecom_airwallex_is_inited) {
                    self::$mecom_airwallex_is_inited = true;
                    add_filter('manage_edit-shop_order_columns', [$this, 'add_mecom_shield_url'], 10, 1);
                    add_filter('manage_shop_order_posts_custom_column', [$this, 'add_mecom_order_values'], 10, 2);
                }
                add_action('admin_enqueue_scripts', array($this, 'admin_scripts'));
                add_action('wp_enqueue_scripts', array($this, 'payment_scripts'));

                // process admin CardsShield Gateway Airwallex
                add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));

                if (!$this->is_valid_for_use()) {
                    $this->enabled = 'no';
                }
            }

            public function add_mecom_shield_url($columns)
            {
                $statusColumnPos = array_search('order_status', array_keys($columns), true);
                $insertPos = false === $statusColumnPos ? count($columns) : $statusColumnPos + 1;

                return array_merge(
                    array_slice($columns, 0, $insertPos),
                    [
                        'mecom_shield_url' => 'Shield URL',
                    ],
                    array_slice($columns, $insertPos)
                );
            }

            public function add_mecom_order_values($column, $wc_order_id)
            {
                $this->add_mecom_shield_url_value($column, $wc_order_id);
            }

            public function add_mecom_shield_url_value($column, $wc_order_id)
            {
                if ('mecom_shield_url' != $column) {
                    return;
                }
                $wc_order = wc_get_order($wc_order_id);

                if (!is_a($wc_order, \WC_Order::class)) {
                    return;
                }
                if ($wc_order->get_payment_method() === 'mecom_airwallex') {
                    echo '[Airwallex] ' . get_post_meta($wc_order_id, METAKEY_AIRWALLEX_PROXY_URL, true);
                }
            }

            /**
             * Plugin options, we deal with it in Step 3 too
             */
            public function init_form_fields()
            {
                $this->form_fields = array(
                    'enabled' => array(
                        'title' => 'Enable/Disable',
                        'label' => 'Enable CardsShield Gateway',
                        'type' => 'checkbox',
                        'description' => '',
                        'default' => 'no'
                    ),
                    'title' => array(
                        'title' => 'Title',
                        'type' => 'text',
                        'description' => '--------------------------------------------------------------',
                        'default' => 'Card',
                        'desc_tip' => false,
                    ),
                    'testmode' => array(
                        'title' => 'Enable/Disable Test Mode',
                        'label' => 'Test Mode',
                        'type' => 'checkbox',
                        'description' => '',
                        'default' => 'yes'
                    ),
                    'payment_notes' => array(
                        'title' => 'Payment notes',
                        'type' => 'textarea',
                        'description' => __('Payment notes are limited to 100 characters, cannot use the special characters.', 'woocommerce-gateway-airwallex'),
                        'default' => 'Please make sure your shipping postal code match with your card billing!',
                        'desc_tip' => true,
                    ),
                    'statement_descriptor' => array(
                        'title' => 'Statement Descriptor',
                        'type' => 'text',
                        'description' => __('Statement descriptors are limited to 22 characters, cannot use the special characters >, <, ", \, \', *, and must not consist solely of numbers. This will appear on your customer\'s statement in capital letters.', 'woocommerce-gateway-airwallex'),
                        'default' => '',
                        'desc_tip' => true,
                        'required' => true,
                    ),
                    'invoice_prefix' => array(
                        'title' => __('Invoice Prefix', 'woocommerce-gateway-airwallex-express-checkout'),
                        'type' => 'text',
                        'description' => __('Please enter a prefix for your invoice numbers.', 'woocommerce-gateway-airwallex-express-checkout'),
                        'default' => 'WC-',
                        'desc_tip' => true,
                        'required' => true,
                    ),
                    'config_proxies_button' => [
                        'id' => 'config_proxies_button',
                        'type' => 'config_proxies_button',
                        'title' => __('Config Shields', 'custom_airwallex'),
                    ],
                    'card_icons' => array(
                        'type' => 'multiselect',
                        'title' => 'Accepted Payment Methods',
                        'class' => 'wc-enhanced-select',
                        'default' => array('visa', 'mastercard', 'american_express'),
                        'options' => array(
                            'visa' => 'Visa',
                            'paypal' => 'Paypal',
                            'mastercard' => 'MasterCard',
                            'jcb' => 'JCB',
                            'discover' => 'Discover',
                            'diners' => 'Diners Club',
                            'american_express' => 'American Express',
                        ),
                        'desc_tip'    => true,
                        'description' => 'The selected icons will show customers which credit card brands you accept.',
                    ),
                    'custom_card_icon_css' => [
                        'title'   => 'Custom Airwallex icon css',
                        'type'    => 'textarea',
                        'default' => '/*
.mecom-airwallex-payment-icon {
    width: 50px;
}
*/',
                        'css' => 'width: 400px; min-height: 110px; resize: both;',
                    ]
                );
            }

            /**
             * Screen button Field
             */
            public function generate_config_proxies_button_html($key, $value)
            {
                ?>
                <tr valign="top">
                    <td colspan="2" class="forminp forminp-<?php echo sanitize_title($value['type']) ?>">
                        <a href="<?php echo admin_url('admin.php?page=mecom-gateway-airwallex'); ?>"
                           class="button"><?php _e('Config Shields', 'custom_airwallex'); ?></a>
                    </td>
                </tr>
                <?php
            }


            /**
             * Check if this gateway is enabled and available in the user's country.
             *
             * @return bool
             */
            public function is_valid_for_use()
            {
                return in_array(
                    get_woocommerce_currency(),
                    apply_filters(
                        'woocommerce_airwallex_supported_currencies',
                        array('AUD', 'BRL', 'CAD', 'MXN', 'NZD', 'HKD', 'SGD', 'USD', 'EUR', 'JPY', 'TRY', 'NOK', 'CZK', 'DKK', 'HUF', 'ILS', 'MYR', 'PHP', 'PLN', 'SEK', 'CHF', 'TWD', 'THB', 'GBP', 'RMB', 'RUB', 'INR')
                    ),
                    true
                );
            }

            /**
             * Get_icon function.
             *
             * @return string
             * @version 4.0.0
             * @since 1.0.0
             */
            public function get_icon()
            {
                 $icons = $this->get_option('card_icons');
                 $icons_str = '';
                 if(is_array($icons)) {
                   foreach ($icons as $icon) {
                        $icons_str = '<img class="mecom-airwallex-payment-icon" src="' . plugins_url('/assets/images/icons/' . $icon . '.svg', __FILE__) . '" style="width: 50px;padding-top: 2px; margin-right: 8px"/>' . $icons_str;                     
                     }  
                 }
                 $icons_str .= '<style type="text/css">' . $this->get_option('custom_card_icon_css') . '</style>';

                 return apply_filters('woocommerce_gateway_icon', $icons_str, $this->id);
            }

            /**
             * Get the transaction URL.
             *
             * @param WC_Order $order
             * @return string
             */
            public function get_transaction_url($order)
            {
                if ($this->testmode) {
                    $this->view_transaction_url = 'https://dashboard.airwallex.com/test/payments/%s';
                } else {
                    $this->view_transaction_url = 'https://dashboard.airwallex.com/payments/%s';
                }
                return parent::get_transaction_url($order);
            }

            /**
             * Load admin scripts.
             *
             * @since 3.3.0
             */
            public function admin_scripts()
            {
                $screen = get_current_screen();
                $screen_id = $screen ? $screen->id : '';

                if ('woocommerce_page_wc-settings' !== $screen_id) {
                    return;
                }

            }

            /**
             * You will need it if you want your custom credit card form, Step 4 is about it
             */
            public function payment_fields()
            {
                global $woocommerce;
                if ($this->description) {
                    echo wpautop(wp_kses_post($this->description));
                }
                $nextProxyUrl = WC()->session->get('mecom-airwallex-proxy-active-url');

                if ($nextProxyUrl) {
                    if (isset($_GET['pay_for_order']) && get_query_var('order-pay')) {
                        mecom_airwallex_generate_input_order();
                    }
                    ?>
                    <iframe id="payment-airwallex-area" referrerpolicy="no-referrer"
                            src="<?= $nextProxyUrl . '?mecom-airwallex-get-payment-form=1' ?>"
                            height="280" frameBorder="0" style="width: 100%"></iframe>
                    <!--                    <iframe style="width: 100%; display: none; position: fixed; top: 0; left: 0; z-index: 99999; height: 100vh"-->
                    <!--                            id="payment-area-airwallex-to-confirm" referrerpolicy="no-referrer"-->
                    <!--                            src="--><? //= $nextProxyUrl . '?mecom-airwallex-get-payment-confirm-form=1' ?><!--"-->
                    <!--                            height="170" frameBorder="0"></iframe>-->
                    <?php
                } else {
                    ?>
                    <div>We cannot accept any payments right now. Please comeback to try tomorrow or select other
                        payment methods if available.
                    </div>
                    <?php
                }
                action_airwallex_wp_footer();

            }

            /*
                * Custom CSS and JS, in most cases required only when you decided to go with a custom credit card form
                */
            public function payment_scripts()
            {
                // we need JavaScript to process a token only on cart/checkout pages, right?
                if (!is_cart() && !is_checkout()) {
                    return;
                }

                // if our payment gateway is disabled, we do not have to enqueue JS too
                if ('no' === $this->enabled) {
                    return;
                }
                wp_register_style('mecom_airwallex_styles', plugins_url('assets/css/styles.css', __FILE__), [], OPT_MECOM_AIRWALLEX_VERSION);
                wp_enqueue_style('mecom_airwallex_styles');

                wp_register_script('mecom_airwallex_js', plugins_url('assets/js/checkout_hook.js', __FILE__), array('jquery'), OPT_MECOM_AIRWALLEX_VERSION, true);
                wp_enqueue_script('mecom_airwallex_js');
                wp_localize_script('mecom_airwallex_js', 'ajax_object', [
                    'cs_add_order_note_nonce_airwallex' => wp_create_nonce('cs_add_order_note_airwallex')
                ]);
            }

            /*
                * Fields validation, more in Step 5
                */
            public function validate_fields()
            {
                return true;
            }

            /*
                * We're processing the payments here, everything about it is in Step 5
                */
            public function getProductTitle($productTitle)
            {
                switch ($this->productTitleSetting) {
                    case 'user_define':
                        return $this->userDefineProductTitle;
                    case 'keep_original':
                        return $productTitle;
                    case 'last_word':
                    default:
                        return strrchr($productTitle, ' ');
                }
            }

            public function get_number_of_decimal_digits()
            {
                return $this->is_currency_supports_zero_decimal() ? 0 : 2;
            }

            public function is_currency_supports_zero_decimal()
            {
                return in_array(get_woocommerce_currency(), array('HUF', 'JPY', 'TWD'));
            }

            public function process_payment($order_id)
            {
                global $woocommerce;
                // we need it to get any order details
                $order = wc_get_order($order_id);

                $activatedProxy = findActivatedProxyDataByIdAirwallex(get_option(Opt_Mecom_Airwallex_Proxies, []), WC()->session->get('mecom-airwallex-proxy-active-id'));

                if (!$activatedProxy) {
                    airwallexLog("Can't find activated proxy!\n");
                    wc_add_notice('We cannot accept any payments right now. Please comeback to try tomorrow or select other payment methods if available.', 'error');
                    return [
                        'result' => 'failure',
                        'reload' => true
                    ];
                }
                // Billing
                $billingFirstName = $order->get_billing_first_name();
                $billingLastName = $order->get_billing_last_name();
                $billingAddress1 = $order->get_billing_address_1();
                $billingAddress2 = $order->get_billing_address_2();
                $billingCity = $order->get_billing_city();
                $billingCountry = $order->get_billing_country();
                $billingPostCode = $order->get_billing_postcode();
                $billingState = $order->get_billing_state();


                // Log processing proxyUrl
                $order->add_order_note(sprintf(__('Starting checkout with Airwallex proxy %s', 'mecom'), $activatedProxy['url']), 0, false);

                $items = [];

                $order_items = $order->get_items();
                foreach ($order_items as $it) {
                    $product = wc_get_product($it->get_product_id());
                    //$product_name = $product->get_name(); // Get the product name
                    $product_name = $this->getProductTitle($product->get_name());

                    $item_quantity = $it->get_quantity(); // Get the item quantity

                    $amount = round($it['line_subtotal'] / $it['qty'], $this->get_number_of_decimal_digits());

                    $items[] = [
                        "name" => $product_name,
                        "quantity" => $item_quantity,
                        "total" => $amount
                    ];
                }
                $paymentIntentIdRequest = null;
                if (!empty($order->get_transaction_id())) {
                    $proxyErrorUrl = get_post_meta($order_id, METAKEY_AIRWALLEX_PROXY_URL_ERROR, true);
                    if ($proxyErrorUrl) {
                        if (WC()->session->get('mecom-airwallex-proxy-active-url') == $proxyErrorUrl) {
                            $paymentIntentIdRequest = $order->get_transaction_id();
                        }
                    } else {
                        $paymentIntentIdRequest = $order->get_transaction_id();
                    }
                }
                $response = wp_remote_post($activatedProxy['url'] . '?mecom-airwallex-make-payment=1&order_id=' . $order->get_id(), [
                    'sslverify' => false,
                    'timeout' => 5 * 60,
                    'headers' => [
                        'Content-Type' => 'application/json',
                    ],
                    'body' => json_encode([
                        'payment_intent' => $paymentIntentIdRequest,
                        'payment_method_id' => $_POST['mecom-airwallex-payment-method-id'],
                        'order_id' => $order->get_id(),
                        'order_invoice' => $this->invoice_prefix . $order->get_order_number(),
                        'order_items' => $items,
                        'statement_descriptor' => $this->get_option('statement_descriptor'),
                        'merchant_site' => get_home_url(),
                        'amount' => $order->get_total(),
                        'customer_zipcode' => $billingPostCode,
                        'customer_email' => $order->get_billing_email(),
                        'currency' => $order->get_currency(),
                        'name' => $order->get_billing_first_name() . ' ' . $order->get_billing_last_name(),
                        'email' => $order->get_billing_email(),
                        'customer' => [
                            'first_name' => $billingFirstName,
                            'last_name' => $billingLastName,
                            'phone' => $order->get_billing_phone(),
                            'address' => [
                                'city' => $billingCity,
                                'country' => $billingCountry,
                                'line1' => $billingAddress1,
                                'line2' => $billingAddress2,
                                'postal_code' => $billingPostCode,
                                'state' => $billingState,
                            ],
                        ]
                    ])
                ]);
                update_post_meta($order->get_id(), METAKEY_AIRWALLEX_PROXY_URL, $activatedProxy['url']);
                update_post_meta($order->get_id(), '_shield_payment_method', 'airwallex');
                update_post_meta($order->get_id(), '_shield_payment_url', $activatedProxy['url']);
                if (is_wp_error($response) || 200 !== wp_remote_retrieve_response_code($response)) {
                    airwallexLog($response, 'Airwallex request error');
                    wc_add_notice('We cannot process your payment right now, please try another payment method.', 'error');
                    return false;
                }
                $body = wp_remote_retrieve_body($response);
                $body = json_decode($body);
                if ($body->status === 'success') {
                    $payment = $body->payment;
                    update_post_meta($order->get_id(), '_transaction_id', $payment->id);
                    WC()->session->set('mecom-airwallex-last-order-id', $order->get_id());
                    return [
                        'result' => 'success',
                        'redirect' => sprintf('#cs-airwallex-confirm-pi-%s:::%s:::%s:::%s', $payment->id, $payment->client_secret, $order_id, uniqid())
                    ];
                } else if ($body->status === 'requires_action') {
                    $paymentIntent = $body->payment_intent;
                    if (isset($body->payment_intent)) {
                        $paymentIntentId = $body->payment_intent->id;
                        update_post_meta($order->get_id(), '_transaction_id', $paymentIntentId);
                        update_post_meta($order->get_id(), METAKEY_AIRWALLEX_PROXY_URL_ERROR, $activatedProxy['url']);
                        $order->add_order_note(sprintf(__('Airwallex charged 3DS require verification by proxy %s, Payment Intent ID: %s', 'mecom'),
                            $activatedProxy['url'],
                            $paymentIntentId
                        ));
                    }
                    if (isset($_GET['pay_for_order'])) {
                        echo json_encode([
                            'result' => 'success',
                            'redirect' => sprintf('#cs-confirm-pi-%s:%s:%s', $paymentIntent->client_secret, $order_id, uniqid()),
                        ]);
                        exit();
                    }

                    return [
                        'result' => 'success',
                        'redirect' => sprintf('#cs-confirm-pi-%s:%s:%s', $paymentIntent->client_secret, $order_id, uniqid()),
                    ];
                } else {
                    airwallexLog($response, 'Airwallex request payment error');
                    // Empty cart
                    $order->update_status('failed');
                    if ($body->code === 'domain_whitelist_not_allow') {
                        $order->add_order_note(sprintf(__('Airwallex charged ERROR by proxy %s, ERROR message: %s', 'mecom'),
                            $activatedProxy['url'],
                            'Domain whitelist is required'
                        ));
                    } else if ($body->code === 'customer_zipcode_not_allow') {
                        $order->add_order_note(sprintf(__('Airwallex charged ERROR by proxy %s, ERROR message: %s', 'mecom'),
                            $activatedProxy['url'],
                            "Customer's zipcode is blacklisted"
                        ));
                        wc_add_notice('The selected payment method is suspended, Please contact merchant for more information.', 'error');
                        return false;
                    } else if ($body->code === 'customer_email_not_allow') {
                        $order->add_order_note(sprintf(__('Airwallex charged ERROR by proxy %s, ERROR message: %s', 'mecom'),
                            $activatedProxy['url'],
                            "Customer's email is blacklisted"
                        ));
                        wc_add_notice('The selected payment method is suspended, Please contact merchant for more information.', 'error');
                        return false;
                    } else if ($body->code === 'states_cities_not_allow') {
                        $order->add_order_note(sprintf(__('Airwallex charged ERROR by proxy %s, ERROR message: %s', 'mecom'),
                            $activatedProxy['url'],
                            "Customer's State and City is blacklisted"
                        ));
                        wc_add_notice('Sorry, Your selected products are not available to purchase due to our policy violation.', 'error');
                        return false;
                    } else if ($body->code === 'order_total_not_allow') {
                        $order->add_order_note(sprintf(__('Airwallex charged ERROR by proxy %s, ERROR message: %s', 'mecom'),
                            $activatedProxy['url'],
                            "Order value exceeds Airwallex capability"
                        ));
                        wc_add_notice('The selected payment method is suspended, Please contact merchant for more information.', 'error');
                        return false;
                    } else {
                        $err = $body->err;
                        $paymentIntentId = $order->get_transaction_id();
                        if (isset($err->payment_intent)) {
                            $paymentIntentId = $err->payment_intent->id;
                            update_post_meta($order->get_id(), '_transaction_id', $paymentIntentId);
                            update_post_meta($order->get_id(), METAKEY_AIRWALLEX_PROXY_URL_ERROR, $activatedProxy['url']);
                        }
                        $order->add_order_note(sprintf(__('Airwallex charged ERROR by proxy %s, ERROR message: %s, Payment Intent ID: %s', 'mecom'),
                            $activatedProxy['url'],
                            is_string($err) ?: $err->message,
                            $paymentIntentId
                        ));
                    }
                    wc_add_notice('We cannot process your payment right now, please try another payment method.[2]', 'error');
                    return false;
                }
            }

            /**
             * Process refund.
             *
             * @param int $order_id Order ID
             * @param float $amount Order amount
             * @param string $reason Refund reason
             *
             * @return boolean True or false based on success, or a WP_Error object.
             */
            public function process_refund($order_id, $amount = null, $reason = '')
            {
                $order = wc_get_order($order_id);
                if (0 == $amount || null == $amount) {
                    return new WP_Error('airwallex_refund_error', __('Refund Error: You need to specify a refund amount.', 'mecom-airwallex-gateway'));
                }

                try {
                    $this->refund_order($order, $order_id, $amount, "", $reason);
                    $order->add_order_note(sprintf(__('Airwallex refund completed; transaction ID = %s', 'mecom-airwallex-gateway'), $order->get_transaction_id()));
                    return true;
                } catch (Exception $e) {
                    airwallexLog($e->getMessage(), 'Airwallex process_refund error');
                    return new WP_Error('airwallex_refund_error', $e->getMessage());
                }
            }

            private function refund_order($order, $order_id, $amount, $refundType, $reason)
            {
                // add refund params
                $params['transaction_id'] = $order->get_transaction_id();
                $params['amount'] = $amount;
                $params['reason'] = $reason;
                $params["merchant_site"] = get_home_url();
                $params['currency'] = $order->get_currency();

                //Get the proxy url when this order was made

                $proxyUrl = get_post_meta($order_id, MetaKey_Airwallex_Proxy_Url, true);

                // do API call
                $url = $proxyUrl . "?mecom-airwallex-refund=1&order_id=$order_id&" . http_build_query($params);

                $request = wp_remote_get($url, [
                   'sslverify' => false,
                ]);

                $notice = 'There is an error when process this payment, please contact us for more support or you can try to use Airwallex!';
                if (is_wp_error($request)) {
                    wc_add_notice($notice, 'error');
                    $order->add_order_note(sprintf(__('Failed refund by Airwallex! Debug proxy %s', 'mecom-airwallex-gateway'), $url));
                    throw new Exception($notice);
                }

                $body = wp_remote_retrieve_body($request);
                $result = json_decode($body);

                if (isset($result->status) && $result->status == "success") {
                    return [
                        'refund_obj' => $result->refund_obj,
                    ];
                } else {
                    $order->add_order_note(sprintf(__('Failed refund by Airwallex! Debug proxy %s', 'mecom-airwallex-gateway'), $url));
                    throw new Exception($notice);
                }
            }
        }
    endif;
}

register_deactivation_hook(__FILE__, 'cs_airwallex_plugin_deactivation');

function cs_airwallex_plugin_deactivation()
{
    wp_clear_scheduled_hook('mecom_gateway_airwallex_daily');
    wp_clear_scheduled_hook('mecom_gateway_airwallex_rotation');
}

//add_action('wp_footer', 'action_airwallex_wp_footer', 10, 1);
function action_airwallex_wp_footer()
{
    if (is_checkout()) {
        $gateways = WC()->payment_gateways->get_available_payment_gateways();
        if (isset($gateways['mecom_airwallex']->enabled) && $gateways['mecom_airwallex']->enabled == 'yes') {
            echo '<div id="cs-airwallex-loader">
                  <div class="cs-airwallex-spinnerWithLockIcon cs-airwallex-spinner" aria-busy="true">
                      <p>We\'re processing your payment...<br/>Please <b>DO NOT</b> close this page!</p>
                  </div>
            </div>';
        }
    }
}

add_action('wp_head', 'action_airwallex_wp_head', 10, 1);

function action_airwallex_wp_head()
{
    if (is_checkout()) {
        $gateways = WC()->payment_gateways->get_available_payment_gateways();
        if (isset($gateways['mecom_airwallex']->enabled) && $gateways['mecom_airwallex']->enabled == 'yes') {
            global $woocommerce;
            if (isEnabledAmountRotationAirwallex()) {
                $nextProxy = getNextProxyAmountRotationAirwallex($woocommerce->cart->get_total(false));
            } else {
                $nextProxy = get_option(Opt_Mecom_Airwallex_Activated_Proxy, null);
            }
            if (empty($nextProxy)) {
                return;
            }
            WC()->session->set('mecom-airwallex-proxy-active-id', $nextProxy['id']);
            WC()->session->set('mecom-airwallex-proxy-active-url', $nextProxy['url']);

            echo '<link rel="preload" href="' . $nextProxy['url'] . '?mecom-airwallex-get-payment-form=1" as="document">';
        }
    }
}

add_action('wc_ajax_cs_add_order_note_airwallex', 'cs_add_order_note_airwallex', 10, 1);

function cs_add_order_note_airwallex()
{
    if (!wp_verify_nonce($_POST['security'], 'cs_add_order_note_airwallex') ||
        empty($_POST['order_id']) || empty($_POST['note'])) {
        die ('Wrong nonce!');
    }
    $order = wc_get_order($_POST['order_id']);
    $order->add_order_note($_POST['note']);
    $order->update_status('failed');
//    $order->save();
    echo json_encode([
        'success' => true
    ]);
    wp_die();
}

function mecom_airwallex_generate_input_order()
{
    $order = wc_get_order(get_query_var('order-pay'));
    $billingFirstName = $order->get_billing_first_name();
    $billingLastName = $order->get_billing_last_name();
    $billingAddress1 = $order->get_billing_address_1();
    $billingAddress2 = $order->get_billing_address_2();
    $billingCity = $order->get_billing_city();
    $billingCountry = $order->get_billing_country();
    $billingPostCode = $order->get_billing_postcode();
    $billingState = $order->get_billing_state();
    ?>
    <input id="billing_first_name" value="<?= $billingFirstName ?>" style="display: none"/>
    <input id="billing_last_name" value="<?= $billingLastName ?>" style="display: none"/>
    <input id="billing_address_1" value="<?= $billingAddress1 ?>" style="display: none"/>
    <input id="billing_address_2" value="<?= $billingAddress2 ?>" style="display: none"/>
    <input id="billing_city" value="<?= $billingCity ?>" style="display: none"/>
    <input id="billing_country" value="<?= $billingCountry ?>" style="display: none"/>
    <input id="billing_postcode" value="<?= $billingPostCode ?>" style="display: none"/>
    <input id="billing_state" value="<?= $billingState ?>" style="display: none"/>
    <div id="mecom_airwallex_pay_for_order_page"></div>
    <?php
}
